using System;
using System.Reflection;
using System.Collections;
using System.Data;
using System.IO;
using System.ComponentModel;
using System.ComponentModel.Design;

namespace RepPP.Toolkit {

    //********************************************************************************
    //
	/// <summary>
	/// Utility functions generics to thick or web clients
	/// </summary>
	//
    //********************************************************************************
	public class Util {

        //********************************************************************************
        //
        /// <summary>
        /// GetCardGroupHierarchy:  Gets the hierarchy of card group using using a leaf.
        /// </summary>
        /// <param name="cgrp">     Leaf card group of the key</param>
        /// <returns>
        /// Array of cardgroups representing the hierarchy terminating with the leaf card group
        /// </returns>
        //
        //********************************************************************************
        static public CardGroup[] GetCardGroupHierarchy(CardGroup cgrp) {
            CardGroup[]     arrRetVal;
            string[]        arrCGrpName;
            int             iCount;

            arrCGrpName = cgrp.GetHierarchy().Split(',');
            iCount      = arrCGrpName.Length;
            arrRetVal   = new CardGroup[iCount];
            for (int iIndex = 0; iIndex < iCount; iIndex++) {
                arrRetVal[iIndex] = cgrp.Card.FindCardGroup(arrCGrpName[iIndex]);
            }
            return(arrRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// BuildPrimaryKey:            Build the primary key of the ancester of the specified card group
        /// </summary>
        /// <param name="grp">          Leaf card group of the key</param>
        /// <param name="bExtValue">    true to build it with external value</param>
        /// <returns>
        /// Array of primary keys of the group
        /// </returns>
        //
        //********************************************************************************
        static public string[] BuildPrimaryKey(Group grp, bool bExtValue) {
            string[]        arrRetVal;
            ArrayList       arrTmp;
            
            arrTmp = new ArrayList();
            foreach (Field fld in grp.Fields) {
                if (fld.IsPrimaryKey) {
                    arrTmp.Add(bExtValue ? fld.ExtValue : fld.Value);
                }
            }
            arrRetVal = new string[arrTmp.Count];
            arrTmp.CopyTo(arrRetVal);
            return(arrRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// BuildFlatHierarchyPrimaryKey:   Build the primary key of the ancester of the specified card group
        /// </summary>
        /// <param name="cgrp">             Leaf card group of the key</param>
        /// <param name="bIncludeSelf">     true to also include the key of the self</param>
        /// <param name="bExtValue">        true to build it with external value</param>
        /// <returns>
        /// Array of cardgroups representing the hierarchy terminating with the leaf card group
        /// </returns>
        //
        //********************************************************************************
        static public string[] BuildFlatHierarchyPrimaryKey(CardGroup cgrp, bool bIncludeSelf, bool bExtValue) {
            string[]        arrRetVal;
            CardGroup[]     arrCardGroup;
            ArrayList       arrTmp;
            
            if (!bIncludeSelf) {
                cgrp = cgrp.Parent as CardGroup;
            }
            arrCardGroup = GetCardGroupHierarchy(cgrp);
            arrTmp       = new ArrayList();
            foreach (CardGroup cgrpNode in arrCardGroup) {
                arrTmp.AddRange(BuildPrimaryKey(cgrpNode.Group, bExtValue));
            }
            arrRetVal = new string[arrTmp.Count];
            arrTmp.CopyTo(arrRetVal);
            return(arrRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// BuildHierarchyPrimaryKey:   Build the primary key of the ancester of the specified card group
        /// </summary>
        /// <param name="cgrp">         Leaf card group of the key</param>
        /// <param name="bIncludeSelf"> true to also include the key of the self</param>
        /// <param name="bExtValue">    true to build it with external value</param>
        /// <returns>
        /// Array of cardgroups representing the hierarchy terminating with the leaf card group
        /// </returns>
        //
        //********************************************************************************
        static public string[][] BuildHierarchyPrimaryKey(CardGroup cgrp, bool bIncludeSelf, bool bExtValue) {
            string[][]      arrRetVal;
            CardGroup[]     arrCardGroup;
            ArrayList       arrTmp;
            int             iIndex;
            
            if (!bIncludeSelf) {
                cgrp = cgrp.Parent as CardGroup;
            }
            arrCardGroup = GetCardGroupHierarchy(cgrp);
            arrRetVal    = new string[arrCardGroup.Length][];
            arrTmp       = new ArrayList();
            iIndex       = 0;
            foreach (CardGroup cgrpNode in arrCardGroup) {
                arrTmp.AddRange(BuildPrimaryKey(cgrpNode.Group, bExtValue));
                arrRetVal[iIndex] = new string[arrTmp.Count];
                arrTmp.CopyTo(arrRetVal[iIndex]);
                arrTmp.Clear();
                iIndex++;
            }
            return(arrRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// BuildPKWhereClause:                 Build a WHERE clause which select an item using its primary key
        /// </summary>
        /// <param name="cgrp">                 Card group</param>
        /// <param name="bBuildAncestralKey">   true to include ancestor's key</param>
        /// <returns>
        /// WHERE clause
        /// </returns>
        //
        //******************************************************************
        public static string BuildPKWhereClause(RepPP.CardGroup cgrp, bool bBuildAncestralKey) {
            string  strRetVal;
            
            if (bBuildAncestralKey && cgrp.Parent != null) {
                strRetVal = BuildPKWhereClause(cgrp.Parent as RepPP.CardGroup, true);
            } else {
                strRetVal = String.Empty;
            }
            foreach (RepPP.Field fld in cgrp.Group.Fields) {
                if (fld.IsPrimaryKey) {
                    if (strRetVal.Length == 0) {
                        strRetVal = "WHERE ";
                    } else {
                        strRetVal += " AND ";
                    }
                    strRetVal += fld.ColName + "=:" + fld.Group.Name + "." + fld.Name;
                }
            }
            return(strRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// BuildPKWhereClause:                 Build a WHERE clause which select an item using its primary key
        /// </summary>
        /// <param name="ginst">                Group instance</param>
        /// <param name="bBuildAncestralKey">   true to include ancestor's key</param>
        /// <returns>
        /// WHERE clause
        /// </returns>
        //
        //******************************************************************
        public static string BuildPKWhereClause(RepPP.GroupInstance ginst, bool bBuildAncestralKey) {
            string  strRetVal;
            
            if (bBuildAncestralKey && ginst.FatherGroupInstance != null) {
                strRetVal = BuildPKWhereClause(ginst.FatherGroupInstance, true);
            } else {
                strRetVal = String.Empty;
            }
            foreach (RepPP.Field fld in ginst.Group.Fields) {
                if (fld.IsPrimaryKey) {
                    if (strRetVal.Length == 0) {
                        strRetVal = "WHERE ";
                    } else {
                        strRetVal += " AND ";
                    }
                    strRetVal += fld.ColName + "=:" + fld.Group.Name + "." + fld.Name;
                }
            }
            return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// BuildArrayOfValues:         Build an array of value of the fields of a group
        /// </summary>
        /// <param name="grp">          Leaf card group of the key</param>
        /// <param name="bExtValue">    true to build it with external value</param>
        /// <returns>
        /// Array of primary keys of the group
        /// </returns>
        //
        //********************************************************************************
        static public string[] BuildArrayOfValues(Group grp, bool bExtValue) {
            ArrayList       arrTemp;
            string[]        arrRetVal;
            
            arrTemp     = new ArrayList(grp.Fields.Count);
            foreach (Field fld in grp.Fields) {
                if (fld.IsVisible) {
                    arrTemp.Add((bExtValue) ? fld.ExtValue : fld.Value);
                }
            }
            arrRetVal =  (string[])arrTemp.ToArray(typeof(string));
            return(arrRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// ReadItemUsingPK:                    Reads an item using the primary key of the item
        /// </summary>
        /// <param name="cinst">                Card instance</param>
        /// <param name="arrKeys">              Array of primary keys</param>
        /// <param name="bAppend">              true to append</param>
        /// <returns>
        /// Returned code
        /// </returns>
        //
        //******************************************************************
        public static int ReadItemUsingPK(CardInstance cinst, string[] arrKeys, bool bAppend) {
            int             iRetVal;
            string          strOldWhereClause;
            CardGroup       cgrpRoot;
            GroupInstance   ginstRoot;
            int             iIndex;
            
            cgrpRoot            = cinst.Card.ChildCardGroups[0];
            ginstRoot           = cinst.RootGroupInstance;
            strOldWhereClause   = cgrpRoot.GetSelectSqlWhere();
            cgrpRoot.SetSelectSqlWhere(BuildPKWhereClause(ginstRoot, false));
            ginstRoot.Empty();
            ginstRoot.AddBlankLine();
            iIndex = 0;
            foreach (Field fld in ginstRoot.Group.Fields) {
                if (fld.IsPrimaryKey) {
                    fld.ExtValue = arrKeys[iIndex++];
                }
            }
            iRetVal = cinst.ReadFromDb(bAppend);
            cgrpRoot.SetSelectSqlWhere(strOldWhereClause);
            return(iRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// FormatStringList:           Format a list of string (prefix str1 suffix,prefix str2 suffix,...)
        /// </summary>
        /// <param name="arrStr">       Array of string</param>
        /// <param name="strPrefix">    Prefix</param>
        /// <param name="strSuffix">    Suffix</param>
        /// <returns>
        /// Formated string (prefix str1 suffix,prefix str2 suffix,...)
        /// </returns>
        //
        //********************************************************************************
        static public string FormatStringList(ArrayList arrStr, string strPrefix, string strSuffix) {
            string strRetVal = String.Empty;;

            foreach (string str in arrStr) {
                if (strRetVal != String.Empty) {
                    strRetVal += ",";
                }
                strRetVal += strPrefix + str + strSuffix;
            }
            return(strRetVal);
        }

        //********************************************************************************
        //
		/// <summary>
		/// IsNumeric:                  Verifies that an object is numeric 
		/// </summary>
		/// <returns>
		/// True if the object is numeric
		/// </returns>
		//
        //********************************************************************************
        static public bool IsNumeric(object num) {
            double result = 0;

            return(Double.TryParse(num.ToString(), 
                                   System.Globalization.NumberStyles.Any, 
                                   System.Globalization.NumberFormatInfo.InvariantInfo, 
                                   out result));
        }

        //********************************************************************************
        //
        /// <summary>
        /// BuildPrimaryKeyFieldNames:         Build an array of the primary key field names
        /// </summary>
        /// <param name="grp">                 Group name</param>
        /// <returns>
        /// Array of primary key field names of the group
        /// </returns>
        //
        //********************************************************************************
        static public string[] BuildPrimaryKeyFieldNames(Group grp) {
            string[]        arrRetVal;
            ArrayList       arrTmp;
            
            arrTmp = new ArrayList();
            foreach (Field fld in grp.Fields) {
                if (fld.IsPrimaryKey) {
                    arrTmp.Add(fld.Name);
                }
            }
            arrRetVal = new string[arrTmp.Count];
            arrTmp.CopyTo(arrRetVal);
            return(arrRetVal);
        }

        //********************************************************************************
        //
		/// <summary>
		/// CopyFieldValues:        Sets the values of the destination fields that do
		///                         exist as source fields.  
		/// </summary>
		//
        //********************************************************************************
        static public void CopyFieldValues(Fields fldsSrc, Fields fldsDest) {
            foreach (Field fld in fldsSrc) {
                if (fldsDest[fld.Name] != null){
                    fldsDest[fld.Name].Value = fld.Value;
                }
            }
        }

        //*********************************************************	    
        //
        /// <summary>
        /// ConvertStringToDateTime:    Convert a string to a DateTime
        /// </summary>
        /// <param name="strVal">       Value to convert</param>
        /// <returns>
        /// DateTime or DateTime.MinValue if error
        /// </returns>
        //
        //*********************************************************
        static public DateTime ConvertStringToDateTime(string strVal) {
            DateTime    dateTimeRetVal = DateTime.MinValue;
            string[]    arrDayTime;
            string[]    arrStr;
            int         iYear;
            int         iMonth;
            int         iDay;
            int         iHour;
            int         iMin;
            int         iSec;

            if (strVal.Length > 9) {
                arrDayTime = strVal.Split(' ');
                if (arrDayTime[0].Length == 10) {
                    arrStr = arrDayTime[0].Split(arrDayTime[0][4]);
                    if (arrStr.Length == 3) {
                        iYear  = Int32.Parse(arrStr[0]);
                        iMonth = Int32.Parse(arrStr[1]);
                        iDay   = Int32.Parse(arrStr[2]);
                        iHour  = 0;
                        iMin   = 0;
                        iSec   = 0;
                        if (arrDayTime.Length > 1) {
                            arrStr = arrDayTime[1].Split(arrDayTime[1][2]);
                            if (arrStr.Length == 3) {
                                iHour   = Int32.Parse(arrStr[0]);
                                iMin    = Int32.Parse(arrStr[1]);
                                iSec    = Int32.Parse(arrStr[2]);
                            }
                        }
                        dateTimeRetVal = new DateTime(iYear, iMonth, iDay, iHour, iMin, iSec);
                    }
                }
            }
            return(dateTimeRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// IsWebDesignMode:               The definite indicator of DesignMode in WebForms
        /// </summary>
        //
        //********************************************************************************
        public static bool IsWebDesignMode() {
            return (System.Web.HttpContext.Current == null);
        }

        //********************************************************************************
        //
        /// <summary>
        /// Handle selectable state of fields of a group
        /// </summary>
        //
        //********************************************************************************
        public class FldSelectableState : IDisposable {
            /// <summary>
            /// Represent the state of the fields of a group
            /// </summary>
            public enum SelStateE {
                /// <summary>No fields for this action should be enabled</summary>
                SS_None     = 0,
                /// <summary>All fields besided primary key for this action should be enabled</summary>
                SS_NoPK     = 1,
                /// <summary>All fields for this action should be enabled</summary>
                SS_All      = 2
            };

            private bool[]          m_arrFieldStates;            // Original selectable state of the field
            private RepPP.Group     m_grp;                      // Group
            private SelStateE       m_eState;                   // Current state

            //********************************************************************************
            //
            /// <summary>
            /// ctor:                           Class constructor
            /// </summary>
            /// <param name="grp">              Group containing the field</param>
            //
            //********************************************************************************
            public FldSelectableState(RepPP.Group grp) {
                int     iIndex;

                m_grp               = grp;
                m_arrFieldStates    = new bool[grp.Fields.Count];
                iIndex              = 0;
                foreach (RepPP.Field fld in m_grp.Fields) {
                    m_arrFieldStates[iIndex++] = fld.IsSelectable;
                }
                m_eState = SelStateE.SS_All;
            }

            //********************************************************************************
            //
            /// <summary>
            /// Dispose:                        Restore original field state
            /// </summary>
            //
            //********************************************************************************
            public void Dispose() {
                int     iIndex;
                
                iIndex          = 0;
                foreach (RepPP.Field fld in m_grp.Fields) {
                    fld.IsSelectable = m_arrFieldStates[iIndex++];
                }
                m_grp = null;
            }

            //********************************************************************************
            //
            /// <summary>
            /// State:                          Current state
            /// </summary>
            //
            //********************************************************************************
            public SelStateE State {
                get {
                    return(m_eState);
                }
                set {
                    int iIndex;
                    
                    if (m_eState != value) {
                        m_eState = value;
                        iIndex   = 0;
                        foreach (RepPP.Field fld in m_grp.Fields) {                            
                            switch(m_eState) {
                            case SelStateE.SS_All:
                                fld.IsSelectable = m_arrFieldStates[iIndex];
                                break;
                            case SelStateE.SS_NoPK:
                                fld.IsSelectable = (fld.IsPrimaryKey) ? false : m_arrFieldStates[iIndex];
                                break;
                            default:
                                fld.IsSelectable = false;
                                break;
                            }
                            iIndex++;
                        }
                    }
                }

            }
        } // Class FldSelectableState
	} // Class Util
}
