using System;
using System.Drawing;
using System.IO;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.ComponentModel;
using System.Collections;
using System.Globalization;
using RepPP.Toolkit.Web;

namespace RepPP.Toolkit.Web {
    //********************************************************************************
    //
    /// <summary>
    /// ToolBar is a web control that groups a list of ToolBarItem objects that are
    /// added declaratively using embedded tags or programatically using the items 
    /// collection.
    /// </summary>
    //
    //********************************************************************************
    [ParseChildren(true,"Items"),
     DefaultProperty("Items"),
     DefaultEvent("Click"),
     ToolboxData("<{0}:ToolBar runat=server></{0}:ToolBar>")]
    public class ToolBar : System.Web.UI.WebControls.WebControl, IPostBackEventHandler {
        /// <summary>
        /// Raised when a user clicks on an item in the toolbar that has a non-empty 
        /// onserverclick attribute
        /// </summary>
        [Browsable(true),
         Category("Action"),
         Description("Event raised when a user clicks on an item in the toolbar that has a non-empty onserverclick attribute")] 
        public event CommandEventHandler    Click;

        /// <summary> Style for enabled item </summary>
        private ToolBarItemStyle            m_styleEnabledItem;
        /// <summary> Style for disabled item</summary>
        private ToolBarItemStyle            m_stylelDisabledItem;
        /// <summary> Style for hover item</summary>
        private ToolBarItemStyle            m_styleHoverItem;
        /// <summary> Style for separator item</summary>
        private ToolBarItemStyle            m_styleSeparator;
        /// <summary> List of toolbar items</summary>
        private ToolBarItemCollection       m_items;

        //********************************************************************************
        //
        /// <summary>
        /// ToolBar:                    ToolBar default constructor
        /// </summary>
        //
        //********************************************************************************
        public ToolBar() {
            m_items          = new ToolBarItemCollection();
        }

        //********************************************************************************
        //
        /// <summary>
        /// Dispose:                        Perform clean up
        /// </summary>
        //
        //********************************************************************************
        public override void Dispose() {
            base.Dispose();
            m_items.Clear();
        }

        //********************************************************************************
        //
        /// <summary>
        /// Items:      Gets the list of ToolBarItem's which is the default inner property used to add
        ///             items declared as embedded tags 
        /// </summary>
        //
        //********************************************************************************
        [DefaultValue(null),
         Category("Data"),
         Description("List of ToolBar Items"),
         NotifyParentProperty(true),
         PersistenceMode(PersistenceMode.InnerDefaultProperty)]
        public virtual ToolBarItemCollection Items {
            get	{
                return(m_items);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// EnabledItemStyle:   Gets the style of the enabled items
        /// </summary>
        //
        //********************************************************************************
        [DefaultValue(null),
         Category("Styles"),
         Description("Style for items when they are enabled"),
         PersistenceMode(PersistenceMode.Attribute),
         DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
        public virtual ToolBarItemStyle EnabledItemStyle {
            get {
                if (m_styleEnabledItem  == null) {
                    m_styleEnabledItem                = new ToolBarItemStyle();
                }
                return(m_styleEnabledItem );
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// DisabledItemStyle:  Gets the style of the disabled items
        /// </summary>
        //
        //********************************************************************************
        [DefaultValue(null), 
         Category("Styles"),
         Description("Style for items when they are disables"),
         PersistenceMode(PersistenceMode.Attribute),
         DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
        public virtual ToolBarItemStyle DisabledItemStyle {
            get {
                if (m_stylelDisabledItem == null) {
                    m_stylelDisabledItem               = new ToolBarItemStyle();
                }
                return(m_stylelDisabledItem);
            }
        }
 
        //********************************************************************************
        //
        /// <summary>
        /// HoverItemStyle:     Gets the style of the items when they are Hovered
        /// </summary>
        //
        //********************************************************************************
        [DefaultValue(null), 
         Category("Styles"),
         Description("Style for items when the mouse pointer is on top of it"),
         PersistenceMode(PersistenceMode.Attribute),
         DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
        public virtual ToolBarItemStyle HoverItemStyle {
            get {
                if (m_styleHoverItem == null) {
                    m_styleHoverItem               = new ToolBarItemStyle();
                }
                return(m_styleHoverItem);
            }
        }
 
        //********************************************************************************
        //
        /// <summary>
        /// SeparatorStyle:     Gets the style of the seperator items
        /// </summary>
        //
        //********************************************************************************
        [DefaultValue(null), 
         Category("Styles"),
         Description("Separator Style"),
         PersistenceMode(PersistenceMode.Attribute),
         DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
        public virtual ToolBarItemStyle SeparatorStyle {
            get {
                if (m_styleSeparator == null) {
                    m_styleSeparator        = new ToolBarItemStyle();
                }
                return(m_styleSeparator);
            }
        }
 
        //********************************************************************************
        //
        /// <summary>
        /// BackImageUrl:       Gets or Sets the url for the background image of the ToolBar
        /// </summary>
        //
        //********************************************************************************
        [Category("Appearance"),
         Description("The background image of the ToolBar"),
         Editor(typeof(System.Web.UI.Design.UrlEditor), typeof(System.Drawing.Design.UITypeEditor))]
        public virtual string BackImageUrl{
            get {
                return(((TableStyle)ControlStyle).BackImageUrl);
            }
            set {
                ((TableStyle)ControlStyle).BackImageUrl = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Gets an item using its id
        /// </summary>
        //`
        //********************************************************************************
        public ToolBarItem this[string strItemId] {
            get {
                ToolBarItem itemRetVal = null;

                foreach (ToolBarItem item in m_items) {
                    if (item.Id.ToUpper() == strItemId.ToUpper()) {
                        itemRetVal = item;
                        break;
                    }
                }
                return(itemRetVal);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnClick:                    Function used to raise the "Click" event when 
        ///                             a toolbar item is clicked.
        /// </summary>
        /// <param name="e">            Event arguments</param>
        //
        //********************************************************************************
        protected virtual void OnClick(CommandEventArgs e) {
            if (Click != null) {
                Click(this, e);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// RaisePostBackEvent:             Implements IPostBackEventHandler.RaisePostBackEvent
        ///                                 enables this server control to process an event 
        ///                                 raised when a form is posted to the server
        /// </summary>
        /// <param name="strEventArgument"> Event arguments</param>
        //
        //********************************************************************************
        public virtual void RaisePostBackEvent(String strEventArgument) {
            ToolBarItem item = Items[strEventArgument];
            if (item.CausesValidation) {
                Page.Validate();
            }
            OnClick(new CommandEventArgs(string.Empty, item.OnServerClick));
        }

        //********************************************************************************
        //
        /// <summary>
        /// CreateControlStyle:         Called the first time the style of the control is 
        ///                             requested. Overridden to create a TableSyle instead
        ///                             of a regular style and also to perform custom 
        ///                             initialization
        /// </summary>
        //
        //********************************************************************************
        protected override Style CreateControlStyle() {
            Style styleRetVal;

            if (ControlStyleCreated){
                styleRetVal = ControlStyle;
            } else {
                styleRetVal = new TableStyle();
            }
            return(styleRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// EnsureDefaultStyles:        Sets the standard styles of the control
        /// </summary>
        //
        //********************************************************************************
        private void EnsureDefaultStyles() {
            TableStyle styleToolBar             = ControlStyle as TableStyle;
            ToolBarItemStyle styleEnabled       = EnabledItemStyle;
            ToolBarItemStyle styleDisablede     = DisabledItemStyle;
            ToolBarItemStyle styleHover         = HoverItemStyle;
            ToolBarItemStyle styleSeparator     = SeparatorStyle;

            if (styleToolBar.BorderStyle == BorderStyle.NotSet &&
                styleToolBar.BorderWidth.IsEmpty               &&
                styleToolBar.BorderColor.IsEmpty               &&
                styleToolBar.BackColor.IsEmpty                 &&
                styleToolBar.BackImageUrl == string.Empty      &&
                styleToolBar.Width.IsEmpty                     &&
                styleToolBar.CssClass == string.Empty) {                          
                styleToolBar.BorderStyle = BorderStyle.Outset;
                styleToolBar.BorderWidth = Unit.Pixel(2);
                styleToolBar.BorderColor = Color.FromArgb(Convert.ToInt32("C3DAF9",16));
                styleToolBar.BackColor   = Color.FromArgb(Convert.ToInt32("C3DAF9",16));
                styleToolBar.Width       = Unit.Percentage(100);
            }
            if (IsEmptyStyle(styleEnabled)) {
                styleEnabled.Font.Names    = new string[]{"Tahoma", "Verdana"};
                styleEnabled.Font.Size     = FontUnit.Point(8);
                styleEnabled.BorderWidth   = Unit.Pixel(0);
                styleEnabled.ForeColor     = Color.Black;
                styleEnabled.VerticalAlign = VerticalAlign.Top;
                styleEnabled.CellPadding   = 2;
            }
            if (IsEmptyStyle(styleDisablede)) {
                styleDisablede.Font.Names    = new string[]{"Tahoma", "Verdana"};
                styleDisablede.Font.Size     = FontUnit.Point(8);
                styleDisablede.BorderWidth   = Unit.Pixel(0);
                styleDisablede.ForeColor     = Color.Gray;
                styleDisablede.VerticalAlign = VerticalAlign.Top;
                styleDisablede.CellPadding   = 2;
            }
            if (IsEmptyStyle(styleHover)) {
                styleHover.Font.Names    = new string[]{"Tahoma", "Verdana"};
                styleHover.Font.Size     = FontUnit.Point(8);
                styleHover.BorderStyle   = BorderStyle.Solid;
                styleHover.BorderWidth   = Unit.Pixel(1);
                styleHover.BorderColor   = ColorTranslator.FromHtml("#336699");
                styleHover.BackColor     = ColorTranslator.FromHtml("#C3DAF9");
                styleHover.VerticalAlign = VerticalAlign.Top;
                styleHover.CellPadding   = 1;
            }
            if (IsEmptyStyle(styleSeparator)) {
                styleSeparator.Height = Unit.Pixel(1);
                styleSeparator.Width  = Unit.Pixel(100);
                //TODO: styleSeparator.CellPadding = 1;
            }

        }

        //********************************************************************************
        //
        /// <summary>
        /// IsEmptyStyle:           Tests if the style was modified in the IDE or not 
        /// </summary>
        /// <param name="style">    Style to be tested</param>
        /// <returns>               True if the style was not modified, false otherwise</returns>
        //
        //********************************************************************************
        private bool IsEmptyStyle(ToolBarItemStyle style) {
            bool bRetVal = false;

            if (style.Font.Names.Length == 0            &&
                style.Font.Size.IsEmpty                 &&
                style.BorderStyle == BorderStyle.NotSet &&
                style.BorderWidth.IsEmpty               &&
                style.BorderColor.IsEmpty               &&
                style.BackColor.IsEmpty                 &&
                style.CssClass == string.Empty) {
                bRetVal = true;
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetStringStyles:                    Extracts the styles or css classes of the toolbar and 
        ///                                     its items
        /// </summary>
        /// <param name="strToolBarStyle">      Toolbar style</param>
        /// <param name="strEnabledStyle">      Style for enabled items</param>
        /// <param name="strDisabledStyle">     Style for disabled items</param>
        /// <param name="strHoverStyle">        Style when items are hovered</param>
        /// <param name="strSeperatorStyle">    Style for separator items</param>
        /// <param name="strToolBarClass">      Style class for the toolbar</param>
        /// <param name="strEnabledClass">      Style class for enabled items</param>
        /// <param name="strDisabledClass">     Style class for disabled items</param>
        /// <param name="strHoverClass">        Style class for hovered items</param>
        /// <param name="strSeperatorClass">    Style class for separator items</param>
        /// <returns>
        /// True if the styles contaion a css class, false otherwise
        /// </returns>
        //
        //********************************************************************************
        private bool GetStylesAndClasses(out string strToolBarStyle,
                                         out string strEnabledStyle,
                                         out string strDisabledStyle,
                                         out string strHoverStyle,
                                         out string strSeperatorStyle,
                                         out string strToolBarClass,
                                         out string strEnabledClass,
                                         out string strDisabledClass,
                                         out string strHoverClass,
                                         out string strSeperatorClass) {
            bool bRetVal = false;

            strToolBarStyle     = WebUtil.RenderStyleInString(this, ControlStyle);
            strEnabledStyle     = WebUtil.RenderStyleInString(this, EnabledItemStyle);
            strDisabledStyle    = WebUtil.RenderStyleInString(this, DisabledItemStyle);
            strHoverStyle       = WebUtil.RenderStyleInString(this, HoverItemStyle);
            strSeperatorStyle   = WebUtil.RenderStyleInString(this, SeparatorStyle);
            if (ControlStyle.CssClass != string.Empty) {
                strToolBarClass     = ControlStyle.CssClass;
                strEnabledClass     = EnabledItemStyle.CssClass;
                strDisabledClass    = DisabledItemStyle.CssClass;
                strHoverClass       = HoverItemStyle.CssClass;
                strSeperatorClass   = SeparatorStyle.CssClass;
            } else {
                strToolBarClass     = string.Empty;
                strEnabledClass     = string.Empty;
                strDisabledClass    = string.Empty;
                strHoverClass       = string.Empty;
                strSeperatorClass   = string.Empty;
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnPreRender:                    Called before rendering this control to 
        ///                                 register the client script. The script is 
        ///                                 included once per page even in the presence
        ///                                 of multiple ToolBar objects
        /// </summary>
        /// <param name="e">                Event arguments</param>
        //
        //********************************************************************************
        protected override void OnPreRender(EventArgs e) {
            const string    strKey = "ToolBarScripts";
            string          strScript;
            StreamReader    reader;

            base.OnPreRender(e);
			if (!WebUtil.IsClientScriptBlockRegistered(Page, strKey)) {
                reader    = new StreamReader(typeof(ToolBar).Assembly.GetManifestResourceStream("WebToolBarScript.js"));
                strScript = "<script langauge='javascript'>\r\n" +
                            "<!--\r\n" + reader.ReadToEnd() +
                            "\r\n-->\r\n</script>\r\n";
				WebUtil.RegisterClientScriptBlock(Page, strKey, strScript);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Render:                         Render this control to the output stream.
        /// </summary>
        /// <param name="output">           The HTML stream writer</param>
        //
        //********************************************************************************
        protected override void Render(HtmlTextWriter output){
            string strEnabledStyle;
            string strDisabledStyle;
            string strHoverStyle;
            string strSeperatorStyle;
            string strToolBarStyle;
            string strEnabledClass;
            string strDisabledClass;
            string strHoverClass;
            string strSeperatorClass;
            string strToolBarClass;
            string strUniqueID;

            EnsureDefaultStyles();
            GetStylesAndClasses(out strToolBarStyle,  
                                out strEnabledStyle,  
                                out strDisabledStyle, 
                                out strHoverStyle,    
                                out strSeperatorStyle,
                                out strToolBarClass,  
                                out strEnabledClass,  
                                out strDisabledClass, 
                                out strHoverClass,    
                                out strSeperatorClass);
            strUniqueID = UniqueID.Replace(":", "$");
            output.Write("<table cellspacing=\"0\" id=\"table_");
            output.Write(strUniqueID);
            output.Write("\" style=\"");
            output.Write(strToolBarStyle);
            output.Write("\" class=\"");
            output.Write(strToolBarClass);
            output.Write("\" EnabledStyle=\"");
            output.Write(strEnabledStyle);
            output.Write("\" EnabledClass=\"");
            output.Write(strEnabledClass);
            output.Write("\" DisabledStyle=\"");
            output.Write(strDisabledStyle);
            output.Write("\" DisabledClass=\"");
            output.Write(strDisabledClass);
            output.Write("\" HoverStyle=\"");
            output.Write(strHoverStyle);
            output.Write("\" HoverClass=\"");
            output.Write(strHoverClass);
            output.WriteLine("\">");
            output.WriteLine("<tr>");
            output.WriteLine("	<td style=\"padding:2;\">");
            output.WriteLine("		<table cellspacing=\"0\">");
            output.WriteLine("		<tr>");
            foreach (ToolBarItem item in m_items) {
                item.Render(output,
                            strUniqueID,
                            WebUtil.GetClientValidatedPostback(Page, this, item.Id),
                            WebUtil.GetPostBackEventReference(Page, this, item.Id),
                            strEnabledStyle,
                            strDisabledStyle,
                            strSeperatorStyle,
                            strEnabledClass,
                            strDisabledClass,
                            strSeperatorClass);
            }
            output.WriteLine("		</tr>");
            output.WriteLine("		</table>");
            output.WriteLine("	</td>");
            output.WriteLine("</tr>");
            output.WriteLine("</table>");
        }

        //********************************************************************************
        //
        /// <summary>
        /// ToolBarItemStyle manages the different style properties used to represent
        /// the ToolBar items for each possible state
        /// </summary>
        //
        //********************************************************************************
        public class ToolBarItemStyle: System.Web.UI.WebControls.TableItemStyle {
            private const int PROP_CELLPADDING = 0x20000;

            //********************************************************************************
            //
            /// <summary>
            /// ToolBarItemStyle :          Class constructor 
            /// </summary>
            //
            //********************************************************************************
            public ToolBarItemStyle() {
            }

            //********************************************************************************
            //
            /// <summary>
            /// ToolBarItemStyle :          Class constructor that accepts a ViewState bag
            ///                             and uses it to initialize the base TableItemStyle
            /// </summary>
            /// <param name="bag">          ViewState bag object</param>
            //
            //********************************************************************************
            public ToolBarItemStyle(StateBag bag): base(bag) {
            }

            //********************************************************************************
            //
            /// <summary>
            /// CellPadding :               Gets or Sets the CellPadding style
            /// </summary>
            //
            //********************************************************************************
            [Bindable(true),
             DefaultValue(-1)]
            public virtual int CellPadding {
                get {
                    int iRetVal = -1;

                    if (base.ViewState["CellPadding"] != null) {
                        iRetVal = (int) base.ViewState["CellPadding"];
                    }
                    return(iRetVal);
                }
                set {
                    if (value < -1) {
                        throw new ArgumentOutOfRangeException("CellPadding value");
                    }
                    base.ViewState["CellPadding"] = value;
                    SetBit(PROP_CELLPADDING);
                }
            }

            //********************************************************************************
            //
            /// <summary>
            /// AddAttributesToRender:  Writes the style attributes of the calling object to
            ///                         the stream
            /// </summary>
            /// <param name="writer">   The html stream</param>
            /// <param name="owner">    the calling control</param>
            //
            //********************************************************************************
            public override void AddAttributesToRender(HtmlTextWriter writer, WebControl owner){
                int iNewValue;

                iNewValue = CellPadding;
                if (iNewValue >= 0) {
                    writer.AddStyleAttribute("padding", 
                                            iNewValue.ToString(NumberFormatInfo.InvariantInfo));
                }
                base.AddAttributesToRender(writer, owner);
            }

            //********************************************************************************
            //
            /// <summary>
            /// CopyFrom :              Duplicates the style properties of the specified 
            ///                         System.Web.UI.WebControls.Style into the instance of 
            ///                         the System.Web.UI.WebControls.TableItemStyle class 
            ///                         that this method is called from.
            /// </summary>
            /// <param name="style">    Represents the style of a Web server control.</param>
            //
            //********************************************************************************
            public override void CopyFrom(Style style) {
                ToolBarItemStyle sourceStyle;

                base.CopyFrom(style);
                if ((style != null)                 && 
                    (style is ToolBarItemStyle)     &&
                    !((ToolBarItemStyle)style).IsEmpty) {
                    sourceStyle = (ToolBarItemStyle)style;
                    CellPadding = sourceStyle.CellPadding;
                }
            }

            //********************************************************************************
            //
            /// <summary>
            /// MergeWith :             Combines the style properties of the specified 
            ///                         System.Web.UI.WebControls.Style into the instance of 
            ///                         the System.Web.UI.WebControls.TableItemStyle class 
            ///                         that this method is called from.  
            /// </summary>
            /// <param name="style">    Represents the style of a Web server control.</param>
            //
            //********************************************************************************
            public override void MergeWith(Style style) {
                ToolBarItemStyle sourceStyle;
     
                if ((style != null) && (style is ToolBarItemStyle) && !((ToolBarItemStyle)style).IsEmpty) {
                    if (IsEmpty) {
                        CopyFrom(style);
                    } else {
                        base.MergeWith(style);
                        sourceStyle = (ToolBarItemStyle)style;
                        if (sourceStyle.CellPadding > -1) {    
                            this.CellPadding = sourceStyle.CellPadding;
                        }
                    }
                } else {
                    base.MergeWith(style);
                }
            }

            //********************************************************************************
            //
            /// <summary>
            /// Reset :                 Removes any defined state elements from the state bag
            /// </summary>
            //
            //********************************************************************************
            public override void Reset(){
                base.ViewState.Remove("CellPadding");
                base.Reset();
            }
        }// class ToolBarItemStyle

        //********************************************************************************
        //
        /// <summary>
        /// ToolBarItemCollection is a strongly typed collection class of ToolBar Items,
        /// It is used mainly to allow us to edit the ToolBar Items using the Collections
        /// Editor provided by the Design Time Environment of VisualStudio.Net
        /// </summary>
        //
        //********************************************************************************
        public class ToolBarItemCollection : CollectionBase { 

            //********************************************************************************
            //
            /// <summary>
            /// this[] :                    Indexer that accesses a ToolBar Item based on its
            ///                             index or unique string id
            /// </summary>
            /// <param name="key">          index or text id</param>
            //
            //********************************************************************************
            public ToolBarItem this[object key] {
                get {
                    return((ToolBarItem)List[IndexOf(key)]); 
                }
                set {
                    List[IndexOf(key)] = value;
                }
            }

            //********************************************************************************
            //
            /// <summary>
            /// Add :                       Adds a ToolBar item to the list
            /// </summary>
            /// <param name="item">         The ToolBarItem to be added</param>
            //
            //********************************************************************************
            public void Add(ToolBarItem item) {
                List.Add(item);
            }

            //********************************************************************************
            //
            /// <summary>
            /// Insert :                    Inserts a ToolBarItem to a specified location
            /// </summary>
            /// <param name="index">        The location of the insertion</param>
            /// <param name="item">         The ToolBarItem to be inserted</param>
            //
            //********************************************************************************
            public void Insert(int index, ToolBarItem item) {
                List.Insert(index,item);
            }

            //********************************************************************************
            //
            /// <summary>
            /// Remove :                    Removes a ToolBarItem from the list
            /// </summary>
            /// <param name="item">          The ToolBarItem to be removed</param>
            //
            //********************************************************************************
            public void Remove(ToolBarItem item) {
                List.Remove(item);
            }

            //********************************************************************************
            //
            /// <summary>
            /// Contains :                  Finds out if a ToolBarItem is in the collection
            /// </summary>
            /// <param name="item">         The ToolBarItem to search for</param>
            //
            //********************************************************************************
            public bool Contains(ToolBarItem item) {
                return List.Contains(item);
            }

            //********************************************************************************
            //
            /// <summary>
            /// IndexOf :                   Finds the index of a TooBarItem in the Collection
            /// </summary>
            /// <param name="obj">          The index or the id of the ToolBarItem</param>
            //
            //********************************************************************************
            public int IndexOf(object obj) {
                int iRetVal = -1;

                if (obj is int){
                    iRetVal = (int)obj;
                } else if (obj is string) {
                    for (int i = 0; i < List.Count; i++){
                        if (String.Compare(((ToolBarItem)List[i]).Id, obj.ToString(), true) == 0){
                            iRetVal = i;
                            break;
                        }
                    }
                } else {
                    throw new ArgumentException("Only a string or an integer is permitted for the indexer.");
                }
                return(iRetVal);
            } 

            //********************************************************************************
            //
            /// <summary>
            /// CopyTo :                    Copies a range of elements from the Collection to
            ///                             a ToolBarItem one dimensional array
            /// </summary>
            /// <param name="array">        The destination array</param>
            /// <param name="index">        The index at which copying begins.</param>
            //
            //********************************************************************************
            public void CopyTo(ToolBarItem[] array, int index) { 
                List.CopyTo(array, index); 
            } 

            //********************************************************************************
            //
            /// <summary>
            /// Contains :                  Finds out if a ToolBarItem having a given id
            ///                             is in the collection
            /// </summary>
            /// <param name="strKey">       ToolBarItem id</param>
            //
            //********************************************************************************
            public bool Contains(string strKey) {
                string  strCurrentKey;
                bool    bRetVal         = false;

                IEnumerator oEnum = GetEnumerator();
                while(oEnum.MoveNext()) {
                    strCurrentKey = ((ToolBarItem)oEnum.Current).Id; 
                    if (string.Compare(strKey, strCurrentKey, true) == 0) {
                        bRetVal = true; 
                    }
                }
                return(bRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// Remove :                    Removes a ToolBarItem having a given id
            ///                             from the  the collection
            /// </summary>
            /// <param name="strKey">       ToolBarItem id</param>
            //
            //********************************************************************************
            public void Remove(string strKey) {
                string strCurrentKey;

                IEnumerator oEnum = this.GetEnumerator();
                while(oEnum.MoveNext()) {
                    strCurrentKey = ((ToolBarItem)oEnum.Current).Id; 
                    if (string.Compare(strKey, strCurrentKey, true) == 0) {
                        Remove((ToolBarItem)oEnum.Current);
                    }
                } 
            } 
        }// class ToolBarItemCollection
	} // Class Toolbar
} // Namespace
