using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Xml;
using System.Resources;
using sdfwwebcsnet;
using sdwebtoolkit;
using System.Web;
using VSLangProj;
using System.IO;
using RepPP;

namespace RepPP.VSNetWizard.ASPNet
{
	/// <summary>
	/// GeneratorBase: Base class for ASP.Net application or item generation
	/// </summary>
	internal abstract class GeneratorBase {
        //*********************************************************     
        //
        /// <summary>
        /// CreateProject:                  Create a new ASP.Net visual studio project
        /// </summary>
        /// <param name="strPrjName">       Project name</param>
        /// <param name="strPrjFullName">   Project full name</param>
        /// <returns>
        /// The created project
        /// </returns>
        //  
        //*********************************************************     
        protected abstract EnvDTE.Project CreateProject(ref string strPrjName, ref string strPrjFullName);

        //*********************************************************     
        //
        /// <summary>
        /// CodeBehindExtension:        Get the code behin extension
        /// </summary>
        //
        //*********************************************************     
        protected abstract string CodeBehindExtension {get;}

        //*********************************************************     
        //
        /// <summary>
        /// GenCardTypedInstance:           Genearte card typed instance 
        /// </summary>
        /// <param name="strPrjName">       Project name</param>
        /// <param name="strCardName">      Card name</param>
        //
        //*********************************************************     
        protected abstract void GenCardTypedInstance(string strPrjName, string strCardName);

        //*********************************************************     
        //
        /// <summary>
        /// GenGroupTypedInstanceV:         Genearte group typed instance 
        /// </summary>
        /// <param name="strPrjName">       Project name</param>
        /// <param name="strGroupName">     Group name</param>
        //
        //*********************************************************     
        protected abstract void GenGroupTypedInstance(string strPrjName, string strGroupName);

        /// <summary>
        /// Define group level for which template can be used to generate the child editor
        /// </summary>
        protected enum ChildEditorGroupLevelE {
            /// <summary>This template should not be used for group editors</summary>
            None            = 0,
            /// <summary>This template should be used only for root group</summary>
            RootOnly        = 1,
            /// <summary>This template should be used only for child groups editors</summary>
            ChildOnly       = 2,
            /// <summary>This template should be used for all groups editors</summary>        
            All             = 3
        }

        protected EnvDTE.DTE                      m_vsApp;                       // Visual studio instance object
        protected RepPP.IApplication              m_app;                         // Rep++ application object
        protected Utility.GenTypedInstParam       m_genTypedInstParam;           // Ttyped instances generation params
        protected string                          m_strXmlTemplFileName;         // Xml Template file name
        protected string                          m_strEditorTemplFileName;      // Template file name for child editor;
        protected string                          m_strRootEditorTemplFileName;  // Template file name for root child editor;
        protected string                          m_strStartPageName;            // name of the start page
        protected Hashtable                       m_hashTemplFiles;              // Collection of template files
        protected Hashtable                       m_hashTemplRefs;               // Collection of template references
        protected Utility.ReplaceFileModeE        m_eReplaceFileMode;            // Replacing file mode
        protected string                          m_strDataConnName;             // Data connection name

        //******************************************************************
        //
        /// <summary>
        /// ctor:                               Class constructor
        /// </summary>
        /// <param name="vsApp">                Visual studio application</param>
        /// <param name="app">                  Rep++ application object</param>
        /// <param name="genTypedInstParam">    Typed instances generation parameters</param>
        /// <param name="strXmlTemplFileName">  Xml template file name</param>
        /// <param name="eReplaceMode">         Replacing file mode</param>
        //
        //******************************************************************
		public GeneratorBase(EnvDTE.DTE                 vsApp,
                             RepPP.IApplication         app,
                             Utility.GenTypedInstParam  genTypedInstParam,
                             string                     strXmlTemplFileName,
                             Utility.ReplaceFileModeE   eReplaceMode) {
            m_vsApp                         = vsApp;
            m_app                           = app;
            m_genTypedInstParam             = genTypedInstParam;
            m_strXmlTemplFileName           = strXmlTemplFileName;
            m_eReplaceFileMode              = eReplaceMode;
            m_strEditorTemplFileName        = String.Empty;
            m_strRootEditorTemplFileName    = String.Empty;
            m_strStartPageName              = String.Empty;
            m_hashTemplFiles                = new Hashtable();
            m_hashTemplRefs                 = new Hashtable();
            m_strDataConnName               = Utility.GetDataConnectionName(m_app.ConnectionName);
            LoadTemplateItems();
            if (m_strEditorTemplFileName == String.Empty && m_strRootEditorTemplFileName != string.Empty) {
                m_strEditorTemplFileName = m_strRootEditorTemplFileName;
            } else if (m_strEditorTemplFileName != String.Empty && m_strRootEditorTemplFileName == string.Empty) {
                m_strRootEditorTemplFileName = m_strEditorTemplFileName;
            } else if (m_strEditorTemplFileName == String.Empty && m_strRootEditorTemplFileName == String.Empty) {
                throw new System.Exception("Cann't find the child editor template page");
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// ResolveFileNames:               Resolve the names of files that match the specified
        ///                                 file name
        /// </summary>
        /// <param name="strFileFullName">  File full name with search pattern</param>
        /// <returns>
        /// Array of files names
        /// </returns>
        //
        //******************************************************************
        private string[] ResolveFileNames(string strFileFullName) {
            string[]    arrRetVal;
            string      strFileName;
            string      strDirName;

            strDirName  = Path.GetDirectoryName(strFileFullName);
            strFileName = Path.GetFileName(strFileFullName);
            arrRetVal   = Directory.GetFiles(strDirName, strFileName);
            return (arrRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// CreateFileInfoFromXmlNode:  Create file info object from an xml file node
        /// </summary>
        /// <returns>
        /// Template file info
        /// </returns>
        //
        //******************************************************************
        private TemplateFileInfo CreateFileInfoFromXmlNode(XmlNode nodeFile) {
            TemplateFileInfo    fileInfoRetVal;
            string              strValue;
            string              strFileName;
            string              strDirName;

            fileInfoRetVal               = new TemplateFileInfo();
            strValue                     = Utility.GetXmlNodeAttributeValue(nodeFile, "fullname", "");
            // If the file name containes a wide character the GetfullPath throw an exception!
            strDirName                   = Path.GetDirectoryName(strValue);
            strDirName                   = Utility.GetFullPath(Path.GetDirectoryName(m_strXmlTemplFileName), strDirName);
            strFileName                  = Path.GetFileName(strValue);
            fileInfoRetVal.FileFullName  = Path.Combine(strDirName, strFileName);
            strValue                     = Utility.GetXmlNodeAttributeValue(nodeFile, "childeditor", "none");
            switch (strValue.ToLower()) {
            case "none":
                fileInfoRetVal.ChildEditorGroupLevel = ChildEditorGroupLevelE.None;
                break;
            case "all":
                fileInfoRetVal.ChildEditorGroupLevel = ChildEditorGroupLevelE.All;
                break;
            case "root":
                fileInfoRetVal.ChildEditorGroupLevel = ChildEditorGroupLevelE.RootOnly;
                break;
            case "child":
                fileInfoRetVal.ChildEditorGroupLevel = ChildEditorGroupLevelE.ChildOnly;
                break;
            }
            strValue                                = Utility.GetXmlNodeAttributeValue(nodeFile, "startpage", "false");
            fileInfoRetVal.IsStartPage              = (strValue.ToLower() == "true") ? true : false;
            strValue                                = Utility.GetXmlNodeAttributeValue(nodeFile, "suffixname", "false");
            fileInfoRetVal.SuffixName               = (strValue.ToLower() == "true") ? true : false;
            strValue                                = Utility.GetXmlNodeAttributeValue(nodeFile, "parsetext", "false");
            fileInfoRetVal.ParseText                = (strValue.ToLower() == "true") ? true : false;
            fileInfoRetVal.DestinationSubDirectory  = Utility.GetXmlNodeAttributeValue(nodeFile, "destinationdir", "");
            strValue                                = Utility.GetXmlNodeAttributeValue(nodeFile, "buildaction", "content");
            switch (strValue.ToLower()) {
            case "none":
                fileInfoRetVal.BuildAction = prjBuildAction.prjBuildActionNone;
                break;
            case "content":
                fileInfoRetVal.BuildAction = prjBuildAction.prjBuildActionContent;
                break;
            case "compile":
                fileInfoRetVal.BuildAction = prjBuildAction.prjBuildActionCompile;
                break;
            case "embeded":
                fileInfoRetVal.BuildAction = prjBuildAction.prjBuildActionEmbeddedResource;
                break;
            }
            return(fileInfoRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// LoadTemplateReferences:          Load the template references
        /// </summary>
        //
        //******************************************************************
        private void LoadTemplateItems() {
            XmlDocument         xmlDoc;
            XmlNode             nodeTemplate;
            XmlNode             nodeRefs;
            XmlNode             nodeFiles;

            xmlDoc = new XmlDocument();
            xmlDoc.Load(m_strXmlTemplFileName);
            nodeTemplate = Utility.FindXmlNode(xmlDoc, null, "Template");
            nodeRefs     = Utility.FindXmlNode(xmlDoc, nodeTemplate, "References");
            nodeFiles    = Utility.FindXmlNode(xmlDoc, nodeTemplate, "Files");
            LoadTemplateReferences(nodeRefs);
            LoadTemplateFiles(nodeFiles);
        }

        //******************************************************************
        //
        /// <summary>
        /// LoadTemplateReferences:         Load the template references
        /// </summary>
        /// <param name="nodeRefs">         Root references node</param>
        //
        //******************************************************************
        private void LoadTemplateReferences(XmlNode nodeRefs) {
            string  strName;
            string  strType;

            foreach (XmlNode node in nodeRefs.ChildNodes) {
                strName = Utility.GetXmlNodeAttributeValue(node, "name", "");
                strType = Utility.GetXmlNodeAttributeValue(node, "type", "local");
                m_hashTemplRefs.Add(strName, strType);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// LoadTemplateFiles:              Load the template files
        /// </summary>
        /// <param name="nodeFiles">        Root files node</param>
        //
        //******************************************************************
        private void LoadTemplateFiles(XmlNode nodeFiles) {
            TemplateFileInfo    fileInfoTmp;
            TemplateFileInfo    fileInfo;
            string[]            arrFileNames;

            foreach (XmlNode nodeFile in nodeFiles.ChildNodes) {
                fileInfoTmp  = CreateFileInfoFromXmlNode(nodeFile);
                arrFileNames = ResolveFileNames(fileInfoTmp.FileFullName);
                foreach (string strFileName in arrFileNames) {
                    fileInfo                         = new TemplateFileInfo(strFileName);
                    fileInfo.ChildEditorGroupLevel   = fileInfoTmp.ChildEditorGroupLevel;
                    fileInfo.IsStartPage             = fileInfoTmp.IsStartPage;
                    fileInfo.ParseText               = fileInfoTmp.ParseText;
                    fileInfo.SuffixName              = fileInfoTmp.SuffixName;
                    fileInfo.BuildAction             = fileInfoTmp.BuildAction;
                    fileInfo.DestinationSubDirectory = fileInfoTmp.DestinationSubDirectory;
                    if (fileInfo.ParseText) {
                        using (StreamReader reader = new StreamReader(fileInfo.FileFullName, System.Text.Encoding.GetEncoding("iso-8859-1"))) {
                            fileInfo.Text = reader.ReadToEnd();
                        }
                    }
                    if (fileInfo.IsStartPage) {
                        m_strStartPageName = fileInfo.FileFullName;
                    }
                    if (fileInfo.ChildEditorGroupLevel != ChildEditorGroupLevelE.None && 
                        Path.GetExtension(fileInfo.FileFullName) == ".aspx") {
                        switch (fileInfo.ChildEditorGroupLevel) {
                        case ChildEditorGroupLevelE.All:
                            m_strEditorTemplFileName     = fileInfo.FileFullName;
                            m_strRootEditorTemplFileName = fileInfo.FileFullName;
                            break;
                        case ChildEditorGroupLevelE.RootOnly:
                            m_strRootEditorTemplFileName = fileInfo.FileFullName;
                            break;
                        case ChildEditorGroupLevelE.ChildOnly:
                            m_strEditorTemplFileName = fileInfo.FileFullName;
                            break;
                        }
                    }
                    if (m_hashTemplFiles[fileInfo.FileFullName] == null) {
                        m_hashTemplFiles.Add(fileInfo.FileFullName, fileInfo);
                    }
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// InternalGenGinstASPNetPage:         Internal method to generate ASP.Net page for a group instance
        /// </summary>
        /// <param name="project">              Visual studio project</param>
        /// <param name="ginst">                Group instance</param>
        /// <param name="strSBCardName">        Selection buffer card name</param>
        /// <param name="strSBRootGroupName">   Selection buffer root group name</param>
        /// <param name="generator">            HTML generator</param>
        /// <param name="bGenChild">            True to generate ASP.Net page for child</param>
        //
        //******************************************************************
        private void InternalGenGinstASPNetPage(EnvDTE.Project      project,
                                                GroupInstance       ginst,
                                                string              strSBCardName,
                                                string              strSBRootGroupName,
                                                GenASPNet           generator,
                                                bool                bGenChild) {
            string          strFullPageName;
            string          strTemplateFileName;
            string          strASPXTemplateText;
            string          strCodeTemplateText;
            RepPP.CardGroup cgrp;

            strFullPageName = Utility.GetProjectDirectoryName(project);
            strFullPageName = Path.Combine(strFullPageName, ginst.Group.Name + ".aspx");
            cgrp            = ginst.CardGroup;
            if (cgrp != null) {
                strTemplateFileName = (cgrp.Card.ChildCardGroups[0] == cgrp) ? m_strRootEditorTemplFileName : m_strEditorTemplFileName;
            } else {
                strTemplateFileName = m_strRootEditorTemplFileName;
            }
            strASPXTemplateText = (m_hashTemplFiles[strTemplateFileName] as TemplateFileInfo).Text;
            strCodeTemplateText = (m_hashTemplFiles[strTemplateFileName + CodeBehindExtension] as TemplateFileInfo).Text;
            GenGroupCodeFile(project.Name, 
                             ginst,
                             strSBCardName,
                             strSBRootGroupName,
                             generator,
                             strCodeTemplateText);
            GenGroupASPXFile(project.Name,
                             ginst,
                             strSBCardName,
                             strSBRootGroupName,
                             generator, 
                             strASPXTemplateText);
            if (bGenChild) {
                foreach (RepPP.GroupInstance ginstChild in ginst.ChildGroupInstances) {
                    InternalGenGinstASPNetPage(project,
                                               ginstChild,
                                               strSBCardName,
                                               strSBRootGroupName,
                                               generator,
                                               bGenChild);
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// GenerateChildEditorPage:            Create child editor for a group
        /// </summary>
        /// <param name="project">              Project</param>
        /// <param name="ginst">                Group instance</param>
        /// <param name="strSBCardName">        Selection buffer card name</param>
        /// <param name="strSBRootGroupName">   Selection buffer root group name</param>
        /// <param name="bGenForChild">         True to generate pages for children</param>
        //
        //******************************************************************
        private void GenerateChildEditorPage(EnvDTE.Project     project,
                                             GroupInstance      ginst,
                                             string             strSBCardName,
                                             string             strSBRootGroupName,
                                             bool               bGenForChild) {
            GenASPNet       genASPNet;
            UserDispatch    disp;

            disp      = new UserDispatch(m_app, Utility.GetProjectDirectoryName(project));
            genASPNet = new GenASPNet(disp, CodeBehindExtension, Utility.GetAssemblyVersion());
            InternalGenGinstASPNetPage(project,
                                       ginst,
                                       strSBCardName,
                                       strSBRootGroupName,
                                       genASPNet,
                                       bGenForChild);
        }

        //******************************************************************
        //
        /// <summary>
        /// SetStartPage:                   Set the start page for the project
        /// </summary>
        /// <param name="strPrjName">       Project name</param>
        /// <param name="strMainCardName">  Transaction card name</param>
        //
        //******************************************************************
        private void SetStartPage(string strPrjName, string strMainCardName) {
            TemplateFileInfo    fileInfo;
            string              strStartPage;

            if (m_strStartPageName != String.Empty) {
                strStartPage = Path.GetFileName(m_strStartPageName);
                if (strStartPage == Path.GetFileName(m_strRootEditorTemplFileName)) {
                    strStartPage = m_app.Cards[strMainCardName].ChildCardGroups[0].Group.Name + ".aspx";
                } else {
                    fileInfo = m_hashTemplFiles[m_strStartPageName] as TemplateFileInfo;
                    if (fileInfo.SuffixName) {
                        strStartPage = Utility.SuffixFileName(strStartPage, strMainCardName);
                    }
                }
                Utility.SetProjectStartPage(m_vsApp, strPrjName, strStartPage);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// GenGroupASPXFile:                   Generate ASPX code for ASP.Net page for a group instance
        /// </summary>
        /// <param name="strPrjName">           Project name</param>
        /// <param name="ginst">                Group instance</param>
        /// <param name="strSBCardName">        Selection buffer card name</param>
        /// <param name="strSBRootGroupName">   Selection buffer root group name</param>
        /// <param name="generator">            HTML generator</param>
        /// <param name="strTemplateText">      Template text</param>
        //
        //******************************************************************
        private void GenGroupASPXFile(string            strPrjName,
                                      GroupInstance     ginst,
                                      string            strSBCardName,
                                      string            strSBRootGroupName,
                                      GenASPNet         generator, 
                                      string            strTemplateText) {
            EnvDTE.Project  project;
            StreamWriter    writer;
            string          strHtml;
            string          strASPXFileName;

            project         = Utility.GetProject(m_vsApp, strPrjName);
            strASPXFileName = Utility.GetProjectDirectoryName(project);
            strASPXFileName = Path.Combine(strASPXFileName, ginst.Group.Name + ".aspx");
            writer          = Utility.CreateTextFile(strASPXFileName, m_eReplaceFileMode);
            if (writer != null) {
                using (writer) {
                    strHtml = generator.GenGinstASPNetPage(strTemplateText,
                                                           ginst,
                                                           ginst.Group.Name,
                                                           GetNamespace(strPrjName),
                                                           strSBCardName,
                                                           strSBRootGroupName,
                                                           m_strDataConnName);
                                                           
                    writer.Write(strHtml);
                }
                project.ProjectItems.AddFromFile(strASPXFileName);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// GetNamespace:                       Get the namespace to use for the generated classes
        /// </summary>
        /// <param name="strPrjName">           Project name</param>
        /// <returns>
        /// The namespace
        /// </returns>
        //
        //******************************************************************
        private string GetNamespace(string strPrjName) {
            string strRetVal;
#if VC8
            strRetVal = Utility.GetProjectDirectoryName(m_vsApp, strPrjName);
            strRetVal = Path.GetFileName(strRetVal);
#else
            strRetVal = strPrjName;
#endif
            return(strRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// GenGroupCodeFile:                   Generate code behind for ASP.Net page for a group instance
        /// </summary>
        /// <param name="strPrjName">           Project name</param>
        /// <param name="ginst">                Group instance</param>
        /// <param name="strSBCardName">        Selection buffer card name</param>
        /// <param name="strSBRootGroupName">   Selection buffer root group name</param>
        /// <param name="generator">            ASP.NET generator</param>
        /// <param name="strTemplateText">      Template text</param>
        //
        //******************************************************************
        private void GenGroupCodeFile(string           strPrjName,
                                      GroupInstance    ginst,
                                      string           strSBCardName,
                                      string           strSBRootGroupName,
                                      GenASPNet        generator, 
                                      string           strTemplateText) {
            EnvDTE.Project  project;
            StreamWriter    writer;
            string          strCode;
            string          strFileName;

            project     = Utility.GetProject(m_vsApp, strPrjName);
            strFileName = Utility.GetProjectDirectoryName(project);
            strFileName = Path.Combine(strFileName, ginst.Group.Name + ".aspx" + CodeBehindExtension);
            writer      = Utility.CreateTextFile(strFileName, m_eReplaceFileMode);
            if (writer != null) {
                using (writer) {
                    strCode = generator.GenGinstASPNetPage(strTemplateText,
                                                           ginst,
                                                           ginst.Group.Name,
                                                           GetNamespace(strPrjName),
                                                           strSBCardName,
                                                           strSBRootGroupName,
                                                           m_strDataConnName);                  
                    writer.Write(strCode);
                }
                project.ProjectItems.AddFromFile(strFileName);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// GenerateGroupItem:             Create ASP.Net group form
        /// </summary>
        /// <param name="strPrjName">       Project name</param>
        /// <param name="strGroupName">     Group name</param>
        //
        //******************************************************************
        public void GenerateGroupItem(string strPrjName, string strGroupName) {
            EnvDTE.Project      project;
            string              strCardName;
            RepPP.Card          crd;
            RepPP.CardInstance  cinst;
            string              strPrjDirName;
#if !VC8
            if (m_genTypedInstParam.m_strNamespace == String.Empty) {
                m_genTypedInstParam.m_strNamespace = strPrjName;
            }
#endif
            project       = Utility.GetProject(m_vsApp, strPrjName);
            strPrjDirName = Utility.GetProjectDirectoryName(project);
            InitProject(strPrjName);
            strCardName = Utility.GetNotDefinedCardName(m_app);
            using (crd = m_app.Cards.Add(strCardName)) {
                crd.ChildCardGroups.Add(m_app.Groups[strGroupName]);
                using (cinst = crd.CardInstances.Add()) {
                    GenerateChildEditorPage(project, cinst.RootGroupInstance, "", "", false);
                }
            }
            GenFilesFromTemplate(strPrjName, strCardName, "");
            if (m_genTypedInstParam.m_eGenType != Utility.GenTypedInstTypeE.None) {
                GenGroupTypedInstance(strPrjName, strGroupName);
            }
            Utility.UpdateInfoResxFile(m_vsApp, strPrjName, m_app, true);
        }

        //******************************************************************
        //
        /// <summary>
        /// GenASPNetFileUsingTemplate:             Generate an ASP.NET file (.aspx or .aspx.cs) using a template
        /// </summary>
        /// <param name="strPrjName">               Project name</param>
        /// <param name="strFileName">              File name</param>
        /// <param name="strClassName">             Class name</param>
        /// <param name="strMainCardName">          Main card name</param>
        /// <param name="strMainRootGrpName">       Root group name for the main card</param>
        /// <param name="strSBCardName">            Selection buffer card name</param>
        /// <param name="strSBGrpName">             Group name for selection buffer</param>
        /// <param name="strTemplateText">          Template text</param>
        /// <param name="eBuildAction">             Build action if none don't include the file to the project</param>
        /// <par
        //
        //******************************************************************
        private void GenASPNetFileUsingTemplate(string          strPrjName,
                                                string          strFileName,
                                                string          strClassName,
                                                string          strMainCardName,
                                                string          strMainRootGrpName,
                                                string          strSBCardName,
                                                string          strSBGrpName,
                                                string          strTemplateText,
                                                prjBuildAction  eBuildAction) {
          
            EnvDTE.Project      project;
            StreamWriter        writer;
            string              strText;
            UserDispatch        disp;
            GenASPNet           parser;
            EnvDTE.ProjectItem  prjItem;

            project     = Utility.GetProject(m_vsApp, strPrjName);
            disp        = new UserDispatch(m_app, Utility.GetProjectDirectoryName(project));
            parser      = new GenASPNet(disp, CodeBehindExtension, Utility.GetAssemblyVersion());            
            strFileName = Path.Combine(Utility.GetProjectDirectoryName(project), strFileName);
            writer      = Utility.CreateTextFile(strFileName, m_eReplaceFileMode);
            if (writer != null) {
                using (writer) {
                    strText = parser.GenASPNetPage(strTemplateText,
                                                   m_app,
                                                   strClassName,
                                                   GetNamespace(strPrjName),
                                                   strMainCardName,
                                                   strMainRootGrpName,
                                                   strSBCardName,
                                                   strSBGrpName,
                                                   m_strDataConnName);
                    writer.Write(strText);
                }
                if (eBuildAction != prjBuildAction.prjBuildActionNone) {
                    prjItem = project.ProjectItems.AddFromFile(strFileName);
#if !VC8
                    prjItem.Properties.Item("BuildAction").Value = eBuildAction;
#endif
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// GenFilesFromTemplate:           Generate files from template
        /// </summary>
        /// <param name="strPrjName">       Project name</param>
        /// <param name="strCardName">      Main card name</param>
        /// <param name="strSBCardName">    Selection buffer card name</param>
        //
        //******************************************************************
        private void GenFilesFromTemplate(string    strPrjName,
                                          string    strCardName,
                                          string    strSBCardName) {
            string              strMainGrpName;
            string              strSBGrpName;
            string              strClassName;
            ArrayList           arrFiles;
            string              strFileName;
            string              strRealFileName;
            string              strDestDir;
            string              strDestFileName;
            TemplateFileInfo    fileInfo;
            EnvDTE.Project      project;
            EnvDTE.ProjectItem  prjItem;

            project  = Utility.GetProject(m_vsApp, strPrjName);
            arrFiles = new ArrayList(m_hashTemplFiles.Keys);
            arrFiles.Sort();
            for (int iIndex = arrFiles.Count - 1; iIndex >= 0; iIndex--) {
                strFileName = (string)arrFiles[iIndex];
                if (strFileName != m_strEditorTemplFileName                          &&
                    strFileName != m_strEditorTemplFileName + CodeBehindExtension    &&
                    strFileName != m_strRootEditorTemplFileName                      &&
                    strFileName != m_strRootEditorTemplFileName + CodeBehindExtension) {
                    strRealFileName = Path.GetFileName(strFileName);
                    fileInfo        = m_hashTemplFiles[strFileName] as TemplateFileInfo;
                    if (fileInfo.SuffixName) {
                        strRealFileName = Utility.SuffixFileName(strRealFileName, strCardName);
                    }
                    if (fileInfo.DestinationSubDirectory != String.Empty) {
                        strRealFileName = Path.Combine(fileInfo.DestinationSubDirectory, strRealFileName);
                    }
                    if (fileInfo.ParseText) {
                        strMainGrpName  = m_app.Cards[strCardName].ChildCardGroups[0].Group.Name;
                        strSBGrpName    = m_app.Cards[strSBCardName].ChildCardGroups[0].Group.Name;
                        strClassName    = Utility.GetFileNameWithoutExtension(Path.GetFileName(strRealFileName));
                        GenASPNetFileUsingTemplate(strPrjName,
                                                   strRealFileName,
                                                   strClassName,
                                                   strCardName,
                                                   strMainGrpName,
                                                   strSBCardName,
                                                   strSBGrpName,
                                                   fileInfo.Text,
                                                   fileInfo.BuildAction);
                    } else {
                        strDestDir = Path.Combine(Utility.GetProjectDirectoryName(project), Path.GetDirectoryName(strRealFileName));
                        if (!Directory.Exists(strDestDir)) {
                            Directory.CreateDirectory(strDestDir);
                        }
                        strDestFileName = Path.Combine(strDestDir, Path.GetFileName(strRealFileName));
                        Utility.CopyFile(fileInfo.FileFullName, strDestFileName, m_eReplaceFileMode);
                        if (fileInfo.BuildAction != prjBuildAction.prjBuildActionNone) {
                            prjItem = project.ProjectItems.AddFromFile(strDestFileName);
#if !VC8
                            prjItem.Properties.Item("BuildAction").Value = fileInfo.BuildAction;
#endif
                        }
                    }
                }
            }            
        }

        //*********************************************************     
        //
        /// <summary>
        /// InitProject:            Initialize the project
        /// </summary>
        /// <param name="project">  Project</param>
        //
        //*********************************************************     
        private void InitProject(string strPrjName) {
            EnvDTE.Project  project;
            
            project = Utility.GetProject(m_vsApp, strPrjName);
            InitProjectRef(project);
            Utility.CreateSessionFolder(Utility.GetProjectDirectoryName(project));
            if (m_genTypedInstParam.m_eGenType != Utility.GenTypedInstTypeE.None) {
                Utility.AddAssemblyAttribute(project, 
                                             "RepPP.AssemblyIncludeTypes",
                                             CodeBehindExtension);
            }
        }
#if VC8
        private void AddWebConfigFile(EnvDTE.Project project) {
            StreamReader    reader;
            StreamWriter    writer;
            string          strText;
            string          strFileName;

            using (reader = new StreamReader(typeof(GeneratorBase).Assembly.GetManifestResourceStream("WebConfig.xml"))) {
                strText = reader.ReadToEnd();
            }
            strFileName = Path.Combine(Utility.GetProjectDirectoryName(project), "Web.Config");
            using (writer = Utility.CreateTextFile(strFileName, Utility.ReplaceFileModeE.sdReplaceWithoutAsk)) {
                writer.Write(strText);
            }
            project.ProjectItems.AddFromFile(strFileName);
        }

        //*********************************************************     
        //
        /// <summary>
        /// AddGACRefToWebSite:             Add a GAC refrence to the web site
        /// </summary>
        /// <param name="project">          Web site project</param>
        /// <param name="strName">          Assembly name</param>
        /// <param name="strVersion">       Assembly version</param>
        /// <param name="strculture">       Assembly culture</param>
        /// <param name="strPublicToken">   Assembly public token</param>
        //
        //*********************************************************     
        private void AddGACRefToWebSite(EnvDTE.Project   project,
                                        string           strName,
                                        string           strVersion,
                                        string           strCulture,
                                        string           strPublicToken) {
            string              strFileName;
            XmlDocument         xmlDoc;
            XmlNode             nodeConfig;
            XmlNode             nodeSystem;
            XmlNode             nodeCompilation;
            XmlNode             nodeAssemblies;
            XmlNode             nodeAssembly;
            string              strValue;

            strFileName = Path.Combine(Utility.GetProjectDirectoryName(project), "Web.Config");
            if (!File.Exists(strFileName)) {
                AddWebConfigFile(project);
            }
            xmlDoc = new XmlDocument();
            xmlDoc.Load(strFileName);
            nodeConfig      = Utility.FindXmlNode(xmlDoc, null,            "configuration");
            nodeSystem      = Utility.FindXmlNode(xmlDoc, nodeConfig,      "system.web");
            nodeCompilation = Utility.FindXmlNode(xmlDoc, nodeSystem,      "compilation");
            nodeAssemblies  = Utility.FindXmlNode(xmlDoc, nodeCompilation, "assemblies");
            strValue        = strName + ", Version=" + strVersion + ", Culture=" + strCulture + ", PublicKeyToken=" + strPublicToken;
            nodeAssembly    = Utility.FindXmlNode(xmlDoc, nodeAssemblies, "add", "assembly", strValue);
            Utility.AddXmlNodeAttribute(xmlDoc, nodeAssembly, "assembly", strValue);
            xmlDoc.Save(strFileName);
        }

        //*********************************************************     
        //
        /// <summary>
        /// AddRefToWebSite:                Add a reference not registred in the GAC to the
        ///                                 web site
        /// </summary>
        /// <param name="project">          Web site project</param>
        /// <param name="strName">          Assembly full name</param>
        //
        //*********************************************************     
        private void AddRefToWebSite(EnvDTE.Project project, string strAssemblyName) {
            string          strBinFolder;
            string          strDestAsemblyName;
            string          strRefreshFileName;
            StreamWriter    writer;

            Utility.AddProjectFolder(project, "Bin");
            strBinFolder       = Path.Combine(Utility.GetProjectDirectoryName(project), "Bin");
            strDestAsemblyName = Path.Combine(strBinFolder, Path.GetFileName(strAssemblyName));
            if (File.Exists(strDestAsemblyName)) {
                File.Delete(strDestAsemblyName);
            }
            File.Copy(strAssemblyName, strDestAsemblyName);
            project.ProjectItems.AddFromFile(strDestAsemblyName);
            strRefreshFileName = strDestAsemblyName + ".refresh";
            using (writer = Utility.CreateTextFile(strRefreshFileName, Utility.ReplaceFileModeE.sdReplaceWithoutAsk)) {
                writer.Write(strAssemblyName);
            }
            project.ProjectItems.AddFromFile(strRefreshFileName);
        }
#endif

        //*********************************************************     
        //
        /// <summary>
        /// InitProjectRef:         Initialize the project refrences
        /// </summary>
        /// <param name="project">  Project</param>
        //
        //*********************************************************     
        private void InitProjectRef(EnvDTE.Project project) {
            string  strBinDir;
            string  strDllName;

            strBinDir  = Utility.GetRepPPBinDir();
            foreach (string strRefName in m_hashTemplRefs.Keys) {
                strDllName = Path.Combine(strBinDir, strRefName);
#if VC8
                if (m_hashTemplRefs[strRefName].ToString().ToUpper() == "GAC") {
                        AddGACRefToWebSite(project,
                                           "sddnet",
                                           "7.2.0.0",
                                           "neutral",
                                           "3A9438D1F064D367");
                } else {
                    AddRefToWebSite(project, strDllName);
                }
#else
                (project.Object as VSLangProj.VSProject).References.Add(strDllName);
#endif
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// GenerateCardItem:               Create card item (HTML forms)
        /// </summary>
        /// <param name="strPrjName">       Project name</param>
        /// <param name="strCardName">      Card name</param>
        /// <param name="strSBCardName">    Selection buffer card name</param>
        //
        //******************************************************************
        public void GenerateCardItem(string    strPrjName, 
                                     string    strCardName,
                                     string    strSBCardName) {
            EnvDTE.Project      project;
            RepPP.Card          crd;
            RepPP.CardInstance  cinst;
            string              strPrjDirName;
            string              strNamespace;

#if !VC8
            if (m_genTypedInstParam.m_strNamespace == String.Empty) {
                m_genTypedInstParam.m_strNamespace = strPrjName;
            }
            strNamespace = strPrjName;
#else
            strNamespace = String.Empty;
#endif
            project       = Utility.GetProject(m_vsApp, strPrjName);
            strPrjDirName = Utility.GetProjectDirectoryName(project);
            InitProject(strPrjName);
            Utility.UpdateInfoResxFile(m_vsApp, strPrjName, m_app, true);
            crd   = m_app.Cards[strCardName];
            cinst = crd.CardInstances.Add();
            using (cinst) {
                GenerateChildEditorPage(project, 
                                        cinst.RootGroupInstance,
                                        strSBCardName,
                                        m_app.Cards[strSBCardName].ChildCardGroups[0].Group.Name,
                                        true);
            } 
            if (m_genTypedInstParam.m_eGenType != Utility.GenTypedInstTypeE.None) {
                GenCardTypedInstance(strPrjName, strCardName);
                if (strSBCardName != null && strSBCardName != String.Empty) {
                    GenCardTypedInstance(strPrjName, strSBCardName);
                }
            }
            GenFilesFromTemplate(strPrjName, strCardName, strSBCardName);
            SetStartPage(strPrjName, strCardName);
        }

        //******************************************************************
        //
        /// <summary>
        /// GenerateApplication:            Create a new web application
        /// </summary>
        /// <param name="strPrjName">       Project name</param>
        /// <param name="strPrjFullName">   Project full name</param>
        /// <param name="strCardName">      Card name</param>
        /// <param name="strSBCardName">    Selection buffer card name</param>
        //
        //******************************************************************
        public void GenerateApplication(ref string      strPrjName, 
                                        ref string      strPrjFullName,
                                        string          strCardName,
                                        string          strSBCardName) {
#if !VC8
            CreateProject(ref strPrjName, ref strPrjFullName);
            if (m_genTypedInstParam.m_strNamespace == String.Empty) {
                m_genTypedInstParam.m_strNamespace = strPrjName;
            }
#endif
            GenerateCardItem(strPrjName, strCardName, strSBCardName);
        }

        //******************************************************************
        //
        /// <summary>
        /// GenASPNet:    Generator class for ASP.Net pages
        /// </summary>
        //
        //******************************************************************
        private class GenASPNet : SCGenHTML {
            protected const string      TAG_CODE_BEHIND_FILE_NAME       = "$$REPPP:CODE_BEHIND_FILE_NAME$$";        // Replaced by code behind file name
            protected const string      TAG_CLASS_NAME                  = "$$REPPP:CLASS_NAME$$";                   // Replaced by class name
            protected const string      TAG_NAMESPACE                   = "$$REPPP:NAMESPACE$$";                    // Replaced by the name space
            protected const string      TAG_FIELD_CONTROL_TYPE          = "$$REPPP:FIELD_CONTROL_TYPE$$";           // Replaced by the field control type 
            protected const string      TAG_MAIN_ROOT_GROUP_NAME        = "$$REPPP:MAIN_ROOT_GROUP_NAME$$";         // Replaced by the root group name of the main card
            protected const string      TAG_SB_ROOT_GROUP_NAME          = "$$REPPP:SB_ROOT_GROUP_NAME$$";           // Replaced by the root group name of the selection buffer card
            protected const string      TAG_VERSION                     = "$$REPPP:VERSION$$";                      // Replaced by rep++ version
            protected const string      TAG_MAIN_CARD_NAME              = "$$REPPP:MAIN_CARD_NAME$$";               // Replaced by the main card name
            protected const string      TAG_SB_CARD_NAME                = "$$REPPP:SB_CARD_NAME$$";                 // Replaced by the selection buffer card name
            protected const string      TAG_TOPPOS_AFTER_FIELDS         = "$$REPPP:TOPPOS_AFTER_FIELDS$$";          // Top position after the field controls
            protected const string      TAG_BEGIN_SET_DATA_CONNECTION   = "$$REPPP:BEGIN_SET_DATA_CONNECTION$$";    // The begin of te data connection setting
            protected const string      TAG_END_SET_DATA_CONNECTION     = "$$REPPP:END_SET_DATA_CONNECTION$$";      // The end of te data connection setting
            protected const string      TAG_DATA_CONNECTION             = "$$REPPP:DATA_CONNECTION$$";             // Data connection name

            /// <summary>Code file extension</summary>
            private string              m_strCodeExt;
            /// <summary>Version of Rep++</summary>
            private string              m_strRepPPVersion;
            /// <summary>Top position for the next field control</summary>
            private int                 m_iNextCtlTop = -1;

            //********************************************************************************
            //
            /// <summary>
            /// ctor:                           Class constructor
            /// </summary>
            /// <param name="disp">             Dispatch object</param>
            /// <param name="strCodeExt">       Code behind extension</param>
            /// <param name="strVersion">       Rep++ version</param>
            //
            //********************************************************************************
            public GenASPNet(SCDispatch disp, string strCodeExt, string strVersion) : base(disp) {
                m_strCodeExt        = strCodeExt;
                m_strRepPPVersion   = strVersion;
            }

            //********************************************************************************
            //
            /// <summary>
            /// Dispose:                            Class destructor
            /// </summary>
            //
            //********************************************************************************
            public override void Dispose() {
                base.Dispose();
            }

            //********************************************************************************
            //
            /// <summary>
            /// ProcessForFieldBlock:               Process the ForEachField block
            /// </summary>
            /// <param name="strText">              Text to process</param>
            /// <param name="ginst">                Group instance</param>
            /// <returns>
            /// Processed text
            /// </returns>
            /// <remarks>
            /// ForEachField    ::= '$$REPPP:FOR_EACH_FIELD_BEGIN$$' {FieldTag} '$$REPPP:FOR_EACH_FIELD_END$$'
            /// FieldTag        ::= '$$REPPP:FIELD_LABEL' | '$$REPPP:FIELD_NAME$$' | '$$REPPP:FIELD_CONTROL$$' | '$REPPP:FIELD_VALUE' | ACTION_TAG
            /// </remarks>
            //                                                                                             
            //********************************************************************************
            protected override string ProcessForEachFieldBlock(string              strText,
	    										               RepPP.GroupInstance ginst) {
                m_iNextCtlTop = -1;
                return(base.ProcessForEachFieldBlock(strText, ginst));
            }

            //********************************************************************************
            //
            /// <summary>
            /// ExtractFieldPosition:       Extract a field position form a string
            /// </summary>
            /// <param name="strParams">    String defining the field parameters tag</param>
            /// <param name="iXPos">        Left position</param>
            /// <param name="iYPos">        Top position</param>
            //
            //********************************************************************************
            private void ExtractFieldPosition(string strParams, ref int iXPos, ref int iYPos) {
                string[]    arrArgs;

				arrArgs = strParams.Split(',');
                if (arrArgs.Length == 2) {
                    iXPos = Convert.ToInt32(arrArgs[0]);
                    if (m_iNextCtlTop != -1) {
                        iYPos = m_iNextCtlTop;
                    } else {
                        iYPos = Convert.ToInt32(arrArgs[1]);
                    }
                } else {
                    // TODO
                    // throw exception
                }
            }

            //********************************************************************************
            //
            /// <summary>
            /// ProcessFieldTag:            Process the field tags
            /// </summary>
            /// <param name="strText">      Text to process</param>
            /// <param name="ginst">        Group instance</param>
            /// <param name="fld">          Field</param>
            /// <returns>
            /// Processed text
            /// </returns>
            /// <remarks>
            /// FieldTagCond    ::= ['$$REPPP:IF_FIELD_ATTR_BEGIN$$''['AttributeList']']'(FieldTag)['$$REPPP:IF_FIELD_ATTR_END$$']
            /// FieldTag        ::= '$$REPPP:FIELD_LABEL' | '$$REPPP:FIELD_NAME$$' | '$$REPPP:FIELD_CONTROL$$' | '$REPPP:FIELD_VALUE' | ACTION_TAG
            /// </remarks>
            //                                                                                             
            //********************************************************************************
            protected override string ProcessFieldTag(string                strText,
                                                      RepPP.GroupInstance   ginst,
                                                      RepPP.Field           fld) {
                string                                      strHtml;
                ASPNetFieldGenerator.ASPNetFldCtlTypeE      eCtlType;
			    int			                                iStart;
			    int			                                iPos;
                string                                      strParams;
                int                                         iLeft   = -1;
                int                                         iTop    = -1;
                int                                         iHeight = -1;
                int                                         iWidth  = -1;
                
                iStart = strText.IndexOf(TAG_FIELD_LABEL);
                if (iStart != -1) {
                    iPos = iStart + TAG_FIELD_LABEL.Length;
                    if (strText[iPos] == '[') {
				        strParams = ExtractString(strText, ref iPos);
                        ExtractFieldPosition(strParams, ref iLeft, ref iTop);
                        iHeight = ASPNetFieldGenerator.GetFieldControlHeight(fld, ASPNetFieldGenerator.ASPNetFldCtlTypeE.sdFldLabel);
                    } else {
                        iLeft   = -1;
                        iTop    = -1;
                        iHeight = -1;
                    }
                    strHtml = ASPNetFieldGenerator.GenASPNetFldCtlText(fld, 
                                                                       ASPNetFieldGenerator.ASPNetFldCtlTypeE.sdFldLabel,
                                                                       ASPNetFieldGenerator.DEF_CTL_PREFIX,
                                                                       iHeight,
                                                                       -1,
                                                                       iLeft,
                                                                       iTop);
                    strText = strText.Substring(0, iStart) + strHtml +  strText.Substring(iPos);
                }
                eCtlType = ASPNetFieldGenerator.GetASPNetFldCtlType(fld);
                iStart   = strText.IndexOf(TAG_FIELD_CONTROL);
                if (iStart != -1) {
                    iPos = iStart + TAG_FIELD_CONTROL.Length;
                    if (strText[iPos] == '[') {
				        strParams = ExtractString(strText, ref iPos);
                        ExtractFieldPosition(strParams, ref iLeft, ref iTop);
                        iHeight       = ASPNetFieldGenerator.GetFieldControlHeight(fld, eCtlType);
                        m_iNextCtlTop = iHeight + iTop + ASPNetFieldGenerator.CTL_VERTICAL_MARGIN;
                    } else {
                        iLeft   = -1;
                        iTop    = -1;
                        if (fld.Type == FieldType.sdFieldMemo) {
                            iHeight = ASPNetFieldGenerator.DEF_MEMO_HEIGHT_PX;
                            iWidth  = ASPNetFieldGenerator.DEF_MEMO_WIDTH_PX;
                        } else {
                            iHeight = -1;
                            iWidth   = -1;
                        }
                    }
                    strHtml  = ASPNetFieldGenerator.GenASPNetFldCtlText(fld, 
                                                                        eCtlType,
                                                                        ASPNetFieldGenerator.DEF_CTL_PREFIX,
                                                                        iHeight,
                                                                        iWidth,
                                                                        iLeft,
                                                                        iTop);
                    strText = strText.Substring(0, iStart) + strHtml +  strText.Substring(iPos);
                }
                strText  = strText.Replace(TAG_FIELD_NAME, fld.Name);
                strText  = strText.Replace(TAG_FIELD_CONTROL_TYPE, ASPNetFieldGenerator.GetASPNetFldCtlName(eCtlType));
                return(strText);
            }

            //********************************************************************************
            //
            /// <summary>
            /// ProcessTopPosAfterFieldsTag:        Process the top position after field controls tag
            /// </summary>
            /// <param name="strText">              Text to process</param>
            /// <returns>
            /// Processed text
            /// </returns>
            /// <remarks>
            /// $$REPPP:TOPPOS_AFTER_FIELDS$$[param]   param is numeric value to add to the
            ///                                        top postion after the field controls 
            /// </remarks>
            //                                                                                             
            //********************************************************************************
            private string ProcessTopPosAfterFieldsTag(string strText) {
                string  strRetVal;
                string  strParam;
                int     iStart;
                int     iPos;
                int     iTop;

                iStart = strText.IndexOf(TAG_TOPPOS_AFTER_FIELDS);
                if (iStart != -1) {
                    iTop = m_iNextCtlTop;
                    iPos = iStart + TAG_TOPPOS_AFTER_FIELDS.Length;
                    if (strText[iPos] == '[') {
				        strParam = ExtractString(strText, ref iPos);
                        iTop += Convert.ToInt32(strParam);
                    }
                    strRetVal = strText.Substring(0, iStart) + iTop +  strText.Substring(iPos);
                } else {
                    strRetVal = strText;
                }
                return(strRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// ProcessDataConnectionTag:       Process the data connection tags
            /// </summary>
            /// <param name="strText">          Text to process</param>
            /// <param name="strDataConnName">  Data connection name</param>
            /// <returns>
            /// Processed text
            /// </returns>
            /// <remarks>
            /// $$REPPP:BEGIN_SET_DATA_CONNECTION$$
            /// $$REPPP:END_SET_DATA_CONNECTION$$
            /// $$REPPP:DATA_CONNECTION$$"
            /// </remarks>
            //                                                                                             
            //********************************************************************************
            protected virtual string ProcessDataConnectionTag(string strText, string strDataConnName) {
                int iIndex1;
                int iIndex2;

                if (strDataConnName != String.Empty) {
                    strText = strText.Replace(TAG_DATA_CONNECTION,           strDataConnName);
                    strText = strText.Replace(TAG_BEGIN_SET_DATA_CONNECTION, "");
                    strText = strText.Replace(TAG_END_SET_DATA_CONNECTION,   "");
                } else {
                    iIndex1 = strText.IndexOf(TAG_BEGIN_SET_DATA_CONNECTION);
                    if (iIndex1 != -1) {
                        iIndex2 = strText.IndexOf(TAG_END_SET_DATA_CONNECTION);
                        if (iIndex2 != -1) {
                            iIndex2 += TAG_BEGIN_SET_DATA_CONNECTION.Length;
                            strText = strText.Replace(strText.Substring(iIndex1, iIndex2 - iIndex1 + 1), "");
                        }
                    }
                }
                return(strText);
            }

            //********************************************************************************
            //
            /// <summary>
            /// ProcessASPNetTag:                   Process tags defined by ASP.NET templates
            /// </summary>
            /// <param name="strText">              Text to process</param>
            /// <param name="strClassName">         Class name</param>
            /// <param name="strNamespace">         Namespace name</param>
            /// <param name="strMainCardName">      Main card name</param>
            /// <param name="strMainRootGroupName"> Root group name of the main card</param>
            /// <param name="strSBCardName">        Selection buffer card name</param>
            /// <param name="strSBRootGroupName">   Root group name of the selection buffer card</param>
            /// <param name="strDataConnName">      Data connection name</param>
            /// <returns>
            /// Processed text
            /// </returns>
            /// <remarks>
            /// $$REPPP:CODE_BEHIND_FILE_NAME$$
            /// $$REPPP:CLASS_NAME$$
            /// $$REPPP:NAMESPACE$$
            /// $$REPPP:START_PAGE$$
            /// $$REPPP:MAIN_ROOT_GROUP_NAME$$
            /// $$REPPP:SB_ROOT_GROUP_NAME$$
            /// $$REPPP:MAIN_CARD_NAME$$
            /// $$REPPP:SB_CARD_NAME$$
            /// $$REPPP:TOPPOS_AFTER_FIELDS$$[param]   param is numeric value to add to the
            ///                                        top postion after the field controls
            /// $$REPPP:BEGIN_SET_DATA_CONNECTION$$
            /// $$REPPP:END_SET_DATA_CONNECTION$$
            /// $$REPPP:DATA_CONNECTION$$"
            /// </remarks>
            //                                                                                             
            //********************************************************************************
            protected virtual string ProcessASPNetTag(string    strText, 
                                                      string    strClassName,
                                                      string    strNamespace,
                                                      string    strMainCardName,
                                                      string    strMainRootGroupName,
                                                      string    strSBCardName,
                                                      string    strSBRootGroupName,
                                                      string    strDataConnName) {
                strText = strText.Replace(TAG_CODE_BEHIND_FILE_NAME, strClassName + ".aspx" + m_strCodeExt);
                strText = strText.Replace(TAG_CLASS_NAME, strClassName);
                strText = strText.Replace(TAG_NAMESPACE, strNamespace);
                strText = strText.Replace(TAG_MAIN_ROOT_GROUP_NAME, strMainRootGroupName);
                strText = strText.Replace(TAG_SB_ROOT_GROUP_NAME, strSBRootGroupName);
                strText = strText.Replace(TAG_VERSION, m_strRepPPVersion);
                strText = strText.Replace(TAG_MAIN_CARD_NAME, strMainCardName);
                strText = strText.Replace(TAG_SB_CARD_NAME, strSBCardName);
                strText = ProcessTopPosAfterFieldsTag(strText);
                strText = ProcessDataConnectionTag(strText, strDataConnName);
                return(strText);
            }

            //********************************************************************************
            //
            /// <summary>
            /// ProcessIsDirty:             Process the IsDirty behavior
            /// </summary>
            /// <param name="strText">      Text to process</param>
            /// <returns>
            /// Processed text
            /// </returns>
            //
            //********************************************************************************
            protected override string ProcessIsDirty(string strText) {
                return(strText);
            }

            //********************************************************************************
            //
		    /// <summary>
		    /// ParseText:                              Parse a text
		    /// </summary>
		    /// <param name="strTemplateText">          Template text</param>
		    /// <param name="ginst">                    Group instance</param>
		    /// <param name="strClassName">             Page class name</param>
		    /// <param name="strNamespace">             Namespace name</param>
		    /// <param name="strMainCardName">          Main card name</param>
		    /// <param name="strMainRootGroupName">     Root group instance name for main card</param>
		    /// <param name="strSBRootGroupName">       Root group name for selection buffer card</param>
            /// <param name="strChildEditorPageName">   Child editor page name for the root group</param>
            /// <param name="strSBPageName">            Selection buffer page name</param>
            /// <param name="strDataConnName">          Data connection name</param>
		    /// <returns>
		    /// Parsed string
		    /// </returns>
            //
            //********************************************************************************
            public string ParseText(string                strTemplateText,
                                    RepPP.GroupInstance   ginst,
                                    string                strClassName,
                                    string                strNamespace,
                                    string                strMainCardName,
                                    string                strMainRootGroupName,
                                    string                strSBCardName,
                                    string                strSBRootGroupName,
                                    string                strDataConnName) {
                string strRetVal;

                if (ginst != null) {
                    if (ginst.LineCount == 0) {
                        ginst.AddBlankLine();
                    }
                    strRetVal = ShowForm(new SCParameters(m_disp.Trace, m_disp),
                                         ginst,
                                         SCCommonUtil.SCFldSelState.SelStateE.SS_None,
                                         false,
                                         true,
                                         "",
                                         null,
                                         null,
                                         "",
                                         null,
                                         strTemplateText,
                                         "");
                } else {
                    strRetVal = strTemplateText;
                }
                strRetVal = ProcessASPNetTag(strRetVal, 
                                             strClassName,
                                             strNamespace,
                                             strMainCardName,
                                             strMainRootGroupName,
                                             strSBCardName,
                                             strSBRootGroupName,
                                             strDataConnName);
                return(strRetVal);
            }

            //********************************************************************************
            //
		    /// <summary>
		    /// GenGinstASPNetPage:                 Generate ASP.Net page for the group instance
		    /// </summary>
		    /// <param name="strTemplateText">      Template text</param>
		    /// <param name="ginst">                Group instance</param>
		    /// <param name="strClassName">         Page class name</param>
		    /// <param name="strNamespace">         Namespace name</param>
		    /// <param name="strSBCardName">        Selection buffer card name</param>
		    /// <param name="strSBRootGroupName">   Selection buffer root group name</param>
            /// <param name="strDataConnName">      Data connection name</param>
		    /// <returns>
		    /// HTML page string
		    /// </returns>
            //
            //********************************************************************************
		    public string GenGinstASPNetPage(string                 strTemplateText,
                                             RepPP.GroupInstance    ginst,
                                             string                 strClassName,
                                             string                 strNamespace,
                                             string                 strSBCardName,
                                             string                 strSBRootGroupName,
                                             string                 strDataConnName) {
                return(ParseText(strTemplateText,
                                 ginst,
                                 strClassName,
                                 strNamespace,
                                 ginst.CardInstance.Card.Name,
                                 ginst.CardInstance.RootGroupInstance.Group.Name,
                                 strSBCardName,
                                 strSBRootGroupName,
                                 strDataConnName));
            }

            //********************************************************************************
            //
		    /// <summary>
		    /// GenASPNetPage:                          Generate ASP.NET page using template
		    /// </summary>
		    /// <param name="strTemplateText">          Template text</param>
		    /// <param name="app">                      Rep++ application</param>
		    /// <param name="strClassName">             Page class name</param>
		    /// <param name="strNamespace">             Namespace name</param>
		    /// <param name="strMainCardName">          Transaction card name</param>
		    /// <param name="strMainRootGroupName">     Root group instance name for main card</param>
		    /// <param name="strSBCardName">            Selection buffer card name</param>
		    /// <param name="strSBRootGroupName">       Root group name for selection buffer card</param>
            /// <param name="strDataConnName">          Data connection name</param>
		    /// <returns>
		    /// HTML page string
		    /// </returns>
            //
            //********************************************************************************
		    public string GenASPNetPage(string              strTemplateText,
                                        RepPP.IApplication  app,
                                        string              strClassName,
                                        string              strNamespace,
                                        string              strMainCardName,
                                        string              strMainRootGroupName,
                                        string              strSBCardName,
                                        string              strSBRootGroupName,
                                        string              strDataConnName) {
                string          strRetVal;
                CardInstance    cinst = null;

                if (app.Cards.Count > 0) {
                    cinst = app.Cards[0].CardInstances.Add();
                }
                try {
                    strRetVal = ParseText(strTemplateText, 
                                          (cinst != null) ? cinst.RootGroupInstance : null,
                                          strClassName,
                                          strNamespace,
                                          strMainCardName,
                                          strMainRootGroupName,
                                          strSBCardName,
                                          strSBRootGroupName,
                                          strDataConnName);
                } finally {
                    if (cinst != null) {
                        cinst.Delete();
                    }
                }
                return(strRetVal);
            }
        } // class GenASPNet

        //********************************************************************************
        //
        /// <summary>
        /// TemplateFileInfo:       Defining a template file info
        /// </summary>
        //
        //********************************************************************************
        private class TemplateFileInfo {
            /// <summary>Full file name</summary>
            private string                  m_strFileFullName;
            /// <summary>Use a suffix when this file is added in the project</summary>
            private bool                    m_bSuffixName;
            /// <summary>Parse this file before adding it to the project</summary>
            private bool                    m_bParseText;
            /// <summary>Child editor template parameter</summary>
            private ChildEditorGroupLevelE  m_eChildEditor;
            /// <summary>This file represents the start page of the project</summary>
            private bool                    m_bStartPage;
            /// <summary>The build action of the file in the project</summary>
            private prjBuildAction          m_eBuildAction;
            /// <summary>The destination subdirectory of the file </summary>
            private string                  m_strDestSubDir;
            /// <summary>The text file </summary>
            private string                  m_strText;

            //********************************************************************************
            //
            /// <summary>
            /// ctor:                           Class constructor
            /// </summary>
            /// <param name="strFullFileName">  File full name</param>
            //
            //********************************************************************************
            public TemplateFileInfo(string strFileFullName) {
                m_strFileFullName   = strFileFullName;
                m_bSuffixName       = false;
                m_bParseText        = false;
                m_eChildEditor      = ChildEditorGroupLevelE.None;
                m_bStartPage        = false;
                m_eBuildAction      = prjBuildAction.prjBuildActionContent;
                m_strDestSubDir     = String.Empty;
                m_strText           = String.Empty;
            }

            //********************************************************************************
            //
            /// <summary>
            /// ctor:                           Class constructor
            /// </summary>
            //
            //********************************************************************************
            public TemplateFileInfo() : this(String.Empty) {
            }

            //********************************************************************************
            //
            /// <summary>
            /// FileFullName:       Get and set the full file name
            /// </summary>
            //
            //********************************************************************************
            public string FileFullName {
                get {
                    return(m_strFileFullName); 
                }
                set {
                    m_strFileFullName = value;
                }
            }

            //********************************************************************************
            //
            /// <summary>
            /// SuffixName:       Get and set whether or not the file name should be suffixed
            /// </summary>
            //
            //********************************************************************************
            public bool SuffixName {
                get {
                    return(m_bSuffixName);
                }
                set {
                    m_bSuffixName = value;
                }
            }

            //********************************************************************************
            //
            /// <summary>
            /// ParseText:       Get and set whether or not the file text should be parsed
            /// </summary>
            //
            //********************************************************************************
            public bool ParseText {
                get {
                    return(m_bParseText);
                }
                set {
                    m_bParseText = value;
                }
            }

            //********************************************************************************
            //
            /// <summary>
            /// ChildEditorGroupLevel:       Get and set the child editor group level
            /// </summary>
            //
            //********************************************************************************
            public ChildEditorGroupLevelE ChildEditorGroupLevel {
                get {
                    return(m_eChildEditor);
                }
                set {
                    m_eChildEditor = value;
                }
            }

            //********************************************************************************
            //
            /// <summary>
            /// IsStartPage:       Get and set whether or not the file is the start page of the project
            /// </summary>
            //
            //********************************************************************************
            public bool IsStartPage {
                get {
                    return(m_bStartPage);
                }
                set {
                    m_bStartPage = value;
                }
            }

            //********************************************************************************
            //
            /// <summary>
            /// BuildAction:       Get and set build action of the file in the project
            /// </summary>
            //
            //********************************************************************************
            public prjBuildAction BuildAction {
                get {
                    return(m_eBuildAction);
                }
                set {
                    m_eBuildAction = value;
                }
            }

            //********************************************************************************
            //
            /// <summary>
            /// DestinationSubDirectory:       Get and set the sub directory where to put the file
            /// </summary>
            //
            //********************************************************************************
            public string DestinationSubDirectory {
                get {
                    return(m_strDestSubDir);
                }
                set {
                    m_strDestSubDir = value;
                }
            }

            //********************************************************************************
            //
            /// <summary>
            /// Text:       Get and set the text file
            /// </summary>
            //
            //********************************************************************************
            public string Text {
                get {
                    return(m_strText);
                }
                set {
                    m_strText = value;
                }
            }
        } // Class TemplateFileInfo
    } // Class GeneratorBase
} // namespace